@@ -1,16 +1,20 @@
 """ CsvPath is the main class for the library. most of the magic
-    happens either here or in individual functions. """
+    happens either here or in individual functions. """  # pylint: disable=C0302
 
-import csv
 import time
 import os
 import hashlib
-from typing import List, Dict, Any
+from datetime import datetime, timezone
+from typing import List, Dict, Any, Self
 from collections.abc import Iterator
 from abc import ABC, abstractmethod
-from csvpath.util.config import Config
-from csvpath.util.line_monitor import LineMonitor
-from csvpath.util.log_utility import LogUtility
+from .util.config import Config
+from .util.line_monitor import LineMonitor
+from .util.log_utility import LogUtility
+from .util.printer import Printer
+from .util.file_readers import DataFileReader
+from .util.line_spooler import LineSpooler, ListLineSpooler
+from .modes.mode_controller import ModeController
 from .matching.matcher import Matcher
 from .scanning.scanner import Scanner
 from .util.metadata_parser import MetadataParser
@@ -25,8 +29,6 @@ from .util.exceptions import (
     CsvPathsException,
 )
 from .matching.util.exceptions import MatchException
-from csvpath.util.printer import Printer
-from csvpath.util.file_readers import CsvDataFileReader
 
 
 class CsvPathPublic(ABC):
@@ -34,7 +36,10 @@ class CsvPathPublic(ABC):
 
     @abstractmethod
     def parse(self, csvpath):  # pragma: no cover
-        """Reads a csvpath prepares to match against CSV file lines"""
+        """Reads a csvpath prepares to match against CSV file lines. This
+        method is an alternative to simply passing the csvpath string to the
+        collect, next, or fast_forward methods. You don't do both.
+        """
 
     @abstractmethod
     def parse_named_path(
@@ -54,24 +59,31 @@ class CsvPathPublic(ABC):
         using the stop() function"""
 
     @abstractmethod
-    def collect(self, nexts: int = -1) -> List[List[Any]]:  # pragma: no cover
-        """Returns the lines of a CSV file that match the csvpath"""
+    def collect(
+        self, csvpath: str = None, *, nexts: int = -1
+    ) -> List[List[Any]]:  # pragma: no cover
+        """Returns the lines of a CSV file that match the csvpath. Pass
+        nexts to limit a run to collecting only N lines; the default
+        is -1 for collecting all. If you do not pass the csvpath
+        string here you must first use the parse method."""
 
     @abstractmethod
     def advance(self, ff: int = -1) -> None:  # pragma: no cover
         """Advances the iteration by ff rows. -1 means to the end of the file."""
 
     @abstractmethod
-    def fast_forward(self) -> None:  # pragma: no cover
+    def fast_forward(self, csvpath: str = None) -> Self:  # pragma: no cover
         """Scans to the end of the CSV file. All scanned rows will be
         considered for match and variables and side effects will happen,
         but no rows will be returned or stored. -1 means to the end of
-        the file."""
+        the file. If you do not pass the csvpath string here you must first
+        use the parse method."""
 
     @abstractmethod
-    def next(self):  # pragma: no cover
+    def next(self, csvpath: str = None):  # pragma: no cover
         """A generator function that steps through the CSV file returning
-        matching rows"""
+        matching rows. If you do not pass the csvpath string here you must
+        first use the parse method."""
 
 
 class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902, R0904
@@ -98,12 +110,17 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
         # a parent CsvPaths may manage a CsvPath instance. if so, it will enable
         # the use of named files and named paths, print capture, error handling,
         # results collection, reference handling, etc. if a CsvPaths is not present
-        # the CsvPath instance is responsible for all its own upkeep and does not have
-        # some of those capabilities.
+        # the CsvPath instance is responsible for all its own upkeep and does not
+        # have some of those capabilities.
         #
         self.csvpaths = csvpaths
         #
+        # modes are set in external comments
         #
+        self.modes = ModeController(self)
+        #
+        # captures the number of lines up front and tracks line stats as the
+        # run progresses
         #
         self._line_monitor = None
         #
@@ -115,27 +132,14 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
         #
         self.match = None
         #
-        # when AND is True we do a logical AND of the match components
-        # to see if there is a match. this is the default. when AND is
-        # False (or set OR to True) the match components are ORed to
-        # determine if a line matches. in the former case all the match
-        # components must agree for a line to match. in the latter case,
-        # if any one match component votes True the line is matched.
-        # technically you can switch from AND to OR, or vice versa, in
-        # the middle of iterating a file using next(). probably not a
-        # good idea, tho.
-        #
-        self._AND = True  # pylint: disable=C0103
-        #
         # when True the lines that do not match are returned from next()
         # and collect(). this effectively switches CsvPath from being an
-        # AND machine to being a NOT AND machine. we do not actually
         # create an OR expression in this case. in the default, we say:
         #     are all of these things true?
         # but when collect_when_not_matched is True we ask:
         #     are any of these things not true?
         #
-        self._when_not_matched = False
+        # self._when_not_matched = False
         self._headers = None
         self.variables: Dict[str, Any] = {}
         self.delimiter = delimiter
@@ -157,16 +161,6 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
         #
         self._freeze_path = False
         #
-        # explain-mode: explain
-        # turns on capturing match reasoning and dumps the captured decisions to INFO
-        # at the end of a match. the reasoning is already present in the DEBUG but it
-        # is harder to see amid all the noise. we don't want to dump explainations
-        # all the time tho because it is very expensive -- potentially 25% worse
-        # performance. the explainations could be improved. atm this is an experimental
-        # feature.
-        #
-        self._explain = False
-        #
         # counts are 1-based
         #
         self.scan_count = 0
@@ -186,6 +180,14 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
         self.stopped = False
         self._advance = 0
         #
+        # the lines var will hold a reference to a LineSpooler instance during a
+        # run, if lines are being collected by the collect() method. if the user
+        # is using this CsvPath instance directly the LineSpooler is only there
+        # as a proxy to the list of lines being collected -- we don't spool to
+        # disk, at least atm.
+        #
+        self.lines = None
+        #
         # set by fail()
         #
         self._is_valid = True
@@ -252,30 +254,6 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
         # way.
         self._config = config
         #
-        # these settings determine how we report function args validation
-        # errors. e.g. if print(True) the validation check fails because
-        # print() expects a string. the more recent trend is to get all
-        # the errors and print statements in the same place controlled by
-        # the same properties. for now this stays because there is a minor
-        # benefit to being able to suppress runtime arg validation and only
-        # use match component rules and exceptions to generate validation
-        # info. but long term this capability may go away.
-        #
-        self._log_validation_errors = True
-        self._print_validation_errors = True
-        self._raise_validation_errors = None
-        self._match_validation_errors = None
-        self._stop_on_validation_errors = None
-        self._fail_on_validation_errors = None
-        #
-        # run mode determines if a csvpath gets run or if it is skipped. the
-        # main reasons to set run-mode: no-run vs. run are: you want to import
-        # it into other csvpaths that are in the same named-paths group, or
-        # you want to switch off a csvpath in a named-paths group for testing
-        # a similar reason.
-        #
-        self._run_mode = True
-        #
         # there are two logger components one for CsvPath and one for CsvPaths.
         # the default levels are set in config.ini. to change the levels pass LogUtility
         # your component instance and the logging level. e.g.:
@@ -284,18 +262,76 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
         self.logger = LogUtility.logger(self)
         self.logger.info("initialized CsvPath")
         self._ecoms = ErrorCommsManager(csvpath=self)
+        #
+        # _function_times_match collects the time a function spends doing its matches()
+        #
         self._function_times_match = {}
+        #
+        # _function_times_value collects the time a function spends doing its to_value()
+        #
         self._function_times_value = {}
+        self._created_at = datetime.now(timezone.utc)
+        self._run_started_at = None
+        self._collecting = False
+        #
+        # holds the unmatched lines when lines are being collected and
+        # _unmatched_available is True. it is analogous to the lines returned
+        # by collect(), but is the lines not returned by collect().
+        #
+        self._unmatched = None
 
     @property
-    def run_mode(self) -> bool:
-        return self._run_mode
+    def data_from_preceding(self) -> bool:
+        return self.modes.source_mode.value
 
-    @run_mode.setter
-    def run_mode(self, mode) -> None:
-        self._run_mode = mode
+    @data_from_preceding.setter
+    def data_from_preceding(self, dfp: bool) -> None:
+        self.modes.source_mode.value = dfp
 
-    def _up_function_time_match(self, c, t) -> None:
+    @property
+    def unmatched(self) -> list[list[Any]]:
+        return self._unmatched
+
+    @unmatched.setter
+    def unmatched(self, lines: list[list[Any]]) -> None:
+        self._unmatched = lines
+
+    @property
+    def collecting(self) -> bool:
+        return self._collecting
+
+    @collecting.setter
+    def collecting(self, c: bool) -> None:
+        self._collecting = c
+
+    @property
+    def unmatched_available(self) -> bool:
+        return self.modes.unmatched_mode.value
+
+    @unmatched_available.setter
+    def unmatched_available(self, ua: bool) -> None:
+        self.modes.unmatched_mode.value = ua
+
+    @property
+    def created_at(self) -> datetime:
+        return self._created_at
+
+    @property
+    def run_started_at(self) -> datetime:
+        return self._run_started_at
+
+    @property
+    def will_run(self) -> bool:
+        return self.modes.run_mode.value
+
+    @will_run.setter
+    def will_run(self, mode) -> None:
+        self.modes.run_mode.value = mode
+
+    #
+    # increases the total accumulated time spent doing c.matches() by t
+    #
+    def up_function_time_match(self, c, t) -> None:
         if c not in self.function_times_match:
             self.function_times_match[c] = 0
         st = self.function_times_match[c]
@@ -306,7 +342,10 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
     def function_times_match(self) -> int:
         return self._function_times_match
 
-    def _up_function_time_value(self, c, t) -> None:
+    #
+    # increases the total accumulated time spent doing c.to_value() by t
+    #
+    def up_function_time_value(self, c, t) -> None:
         if c not in self.function_times_value:
             self.function_times_value[c] = 0
         st = self.function_times_value[c]
@@ -350,44 +389,55 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
 
     @property
     def AND(self) -> bool:  # pylint: disable=C0103
-        return self._AND
+        return self.modes.logic_mode.value
 
     @AND.setter
     def AND(self, a: bool) -> bool:  # pylint: disable=C0103
-        self._AND = a
+        self.modes.logic_mode.value = a
 
     @property
     def OR(self) -> bool:  # pylint: disable=C0103
-        return not self._AND
+        return not self.modes.logic_mode.value
 
     @OR.setter
     def OR(self, a: bool) -> bool:  # pylint: disable=C0103
-        self._AND = not a
+        self.modes.logic_mode.value = not a
 
     @property
     def identity(self) -> str:
-        """returns id or name if found in metadata. the id or name gets
-        into metadata primarily if found in an "external" comment in
-        the csvpath. "external" meaning outside the []s. comments are
-        keyword:comment. we take id, Id, ID and name, Name, NAME. id
-        is preferred over name. E.g.:
-        ~ name: my path description: an example ~
+        """returns id or name if found in metadata.
+
+        the id or name gets into metadata primarily if found
+        in an "external" comment in the csvpath. "external"
+        meaning outside the []s. comments are keyword:comment.
+        we take id, Id, ID and name, Name, NAME.
+
+        id is preferred over name. E.g. in:
+        ~ name: my path description: an example id: this value wins ~
+        the id becomes the identity of the instance.
+
+        we prefer in this order: all-lower most, Initial-caps,
+        ALL-CAPS least
+
+        the ordering is relied on in Result and possibly
+        elsewhere.
         """
+        ret = None
         if not self.metadata:
-            return ""
-        if "id" in self.metadata:
-            return self.metadata["id"]
-        if "Id" in self.metadata:
-            return self.metadata["Id"]
-        if "ID" in self.metadata:
-            return self.metadata["ID"]
-        if "name" in self.metadata:
-            return self.metadata["name"]
-        if "Name" in self.metadata:
-            return self.metadata["Name"]
+            ret = ""
         if "NAME" in self.metadata:
-            return self.metadata["NAME"]
-        return ""
+            ret = self.metadata["NAME"]
+        if "Name" in self.metadata:
+            ret = self.metadata["Name"]
+        if "name" in self.metadata:
+            ret = self.metadata["name"]
+        if "ID" in self.metadata:
+            ret = self.metadata["ID"]
+        if "Id" in self.metadata:
+            ret = self.metadata["Id"]
+        if "id" in self.metadata:
+            ret = self.metadata["id"]
+        return ret
 
     @property
     def config(self) -> Config:  # pylint: disable=C0116
@@ -395,7 +445,7 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
             self._config = Config()
         return self._config
 
-    def has_errors(self) -> bool:  # pylint: disable=C0116
+    def has_errors(self) -> bool:
         if self.errors and len(self.errors) > 0:
             return True
         if self.error_collector:
@@ -431,86 +481,29 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
                 self._errors = []
             self._errors.append(error)
 
-    #
-    # validation error handling overrides the error policy in config. this
-    # is because the validation handling is:
-    #   - different. it is built-in. it deals with programmatic decisions
-    #     about how functions work and is the basis for structural (schema)
-    #     validation.
-    #   - set on a per csvpath basis in comments
-    #
-    # by default, validation errors do not impact matching. they are print
-    # and raise only. however, you can set them to raise or match/not-match
-    # and/or suppress printing.
-    #
-    def set_validation_error_handling(self, veh) -> None:
-        # print prints to the Printer(s), not std.out. atm, no
-        # customization of messages is possible, so there is likely
-        # to be stylistic mismatch with other output.
-        if veh and veh.find("no-print") > -1:
-            self._print_validation_errors = False
-        elif veh and veh.find("print") > -1:
-            self._print_validation_errors = True
-        else:
-            self._print_validation_errors = None
-        #
-        if veh and veh.find("no-raise") > -1:
-            self._raise_validation_errors = False
-        elif veh and veh.find("raise") > -1:
-            self._raise_validation_errors = True
-        else:
-            self._raise_validation_errors = None
-        #
-        # match, no-match, and None do:
-        #   match: return True on error
-        #   no-match: return False on error
-        #   None: default behavior: default_match() or result of matches()
-        if veh and veh.find("no-match") > -1:
-            self._match_validation_errors = False
-        elif veh and veh.find("match") > -1:
-            self._match_validation_errors = True
-        else:
-            self._match_validation_errors = None
-        #
-        # also stop and fail to match the config
-        #
-        if veh and veh.find("no-stop") > -1:
-            self._stop_on_validation_errors = False
-        elif veh and veh.find("stop") > -1:
-            self._stop_on_validation_errors = True
-        else:
-            self._stop_on_validation_errors = None
-        #
-        if veh and veh.find("no-fail") > -1:
-            self._fail_on_validation_errors = False
-        elif veh and veh.find("fail") > -1:
-            self._fail_on_validation_errors = True
-        else:
-            self._fail_on_validation_errors = None
-
     @property
     def stop_on_validation_errors(self) -> bool:
-        return self._stop_on_validation_errors
+        return self.modes.validation_mode.stop_on_validation_errors
 
     @property
     def fail_on_validation_errors(self) -> bool:
-        return self._fail_on_validation_errors
+        return self.modes.validation_mode.fail_on_validation_errors
 
     @property
     def print_validation_errors(self) -> bool:
-        return self._print_validation_errors
+        return self.modes.validation_mode.print_validation_errors
 
     @property
     def log_validation_errors(self) -> bool:
-        return self._log_validation_errors
+        return self.modes.validation_mode.log_validation_errors
 
     @property
     def raise_validation_errors(self) -> bool:
-        return self._raise_validation_errors
+        return self.modes.validation_mode.raise_validation_errors
 
     @property
     def match_validation_errors(self) -> bool:
-        return self._match_validation_errors
+        return self.modes.validation_mode.match_validation_errors
 
     def add_printer(self, printer) -> None:  # pylint: disable=C0116
         if printer not in self.printers:
@@ -523,7 +516,7 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
     def has_default_printer(self) -> bool:
         if not self.printers:
             self.printers = []
-        for i, p in enumerate(self.printers):
+        for p in self.printers:
             if isinstance(p, StdOutPrinter):
                 return True
         return False
@@ -538,15 +531,17 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
 
     @property
     def last_line(self):
+        """this method only returns the default printer's last_line"""
         if not self.printers or len(self.printers) == 0:
             return None
         return self.printers[0].last_line
 
     @property
     def lines_printed(self) -> int:
+        """this method only returns the default printer's lines printed"""
         if not self.printers or len(self.printers) == 0:
             return -1
-        self.printers[0].lines_printed
+        return self.printers[0].lines_printed
 
     @property
     def is_frozen(self) -> bool:
@@ -565,24 +560,24 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
         to INFO. this can be expensive. a 25% performance hit wouldn't
         be unexpected.
         """
-        return self._explain
+        return self.modes.explain_mode.value
 
     @explain.setter
     def explain(self, yesno: bool) -> None:
-        self._explain = yesno
+        self.modes.explain_mode.value = yesno
 
     @property
     def collect_when_not_matched(self) -> bool:
         """when this property is True CsvPath returns the lines that do not
         match the matchers match components"""
-        return self._when_not_matched
+        return self.modes.return_mode.collect_when_not_matched
 
     @collect_when_not_matched.setter
     def collect_when_not_matched(self, yesno: bool) -> None:
-        """when collect_when_not_matched is True we return the lines that failed
+        """when c ollect_when_not_matched is True we return the lines that failed
         to match, rather than the default behavior of returning the matches.
         """
-        self._when_not_matched = yesno
+        self.modes.return_mode.collect_when_not_matched = yesno
 
     def parse(self, csvpath, disposably=False):
         """displosably is True when a Matcher is needed for some purpose other than
@@ -641,97 +636,71 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
         #   - return-mode: matches | no-matches
         #   - print-mode: default | no-default
         #   - validation-mode: (no-)print | log | (no-)raise | quiet | (no-)match
+        #   - run-mode: no-run | run
+        #   - unmatched-mode: no-keep | keep
+        #   - source-mode: preceding | origin
+        #   - files-mode: all | no-data | no-unmatched | no-printouts | data | unmatched | errors | meta | vars | printouts
         #
-        self.update_logic_mode_if()
-        self.update_run_mode_if()
-        self.update_match_mode_if()
-        self.update_print_mode_if()
-        self.update_explain_mode_if()
-        self.update_arg_validation_mode_if()
+        self.modes.update()
 
-    def update_arg_validation_mode_if(self) -> None:
-        if self.metadata and "validation-mode" in self.metadata:
-            # sets arg validation reporting. one or more or none of:
-            #  - print
-            #  - log
-            #  - raise
-            #
-            validation_mode = f"{self.metadata['validation-mode']}".strip()
-            if validation_mode:
-                self.set_validation_error_handling(validation_mode)
-                self.logger.info(
-                    "Setting 'validation-mode': %s",
-                    self.metadata["validation-mode"],
-                )
+    # =====================
+    # in principle the modes should come through the mode controller like:
+    #      self.modes.transfer_mode.value
+    # not wading into that today. low value.
+    #
+    @property
+    def transfer_mode(self) -> str:
+        return self.metadata.get("transfer-mode")
 
-    def update_run_mode_if(self) -> None:
-        if self.metadata and "run-mode" in self.metadata:
-            if f"{self.metadata['run-mode']}".strip() == "no-run":
-                self.run_mode = False
-            elif f"{self.metadata['run-mode']}".strip() == "run":
-                self.run_mode = True
-            else:
-                self.logger.warning(
-                    "Incorrect metadata field value 'run-mode': %s",
-                    self.metadata["run-mode"],
-                )
+    @property
+    def source_mode(self) -> str:
+        return self.metadata.get("source-mode")
 
-    def update_logic_mode_if(self) -> None:
-        if self.metadata and "logic-mode" in self.metadata:
-            if f"{self.metadata['logic-mode']}".strip() == "AND":
-                self.AND = True
-            elif f"{self.metadata['logic-mode']}".strip() == "OR":
-                self.AND = False
-            else:
-                self.logger.warning(
-                    "Incorrect metadata field value 'logic-mode': %s",
-                    self.metadata["logic-mode"],
-                )
+    @property
+    def files_mode(self) -> str:
+        return self.metadata.get("files-mode")
 
-    def update_match_mode_if(self) -> None:
-        if "return-mode" in self.metadata:
-            if f"{self.metadata['return-mode']}".strip() == "matches":
-                self.collect_when_not_matched = False
-            elif f"{self.metadata['return-mode']}".strip() == "no-matches":
-                self.collect_when_not_matched = True
-            else:
-                self.logger.warning(
-                    "Incorrect metadata field value 'return-mode': %s",
-                    self.metadata["return-mode"],
-                )
+    @property
+    def validation_mode(self) -> str:
+        return self.metadata.get("validation-mode")
 
-    def update_explain_mode_if(self) -> None:
-        if "explain-mode" in self.metadata:
-            if f"{self.metadata['explain-mode']}".strip() == "no-explain":
-                self._explain = False
-            elif f"{self.metadata['explain-mode']}".strip() == "explain":
-                self._explain = True
-            else:
-                self._explain = False
+    @property
+    def run_mode(self) -> str:
+        return self.metadata.get("run-mode")
 
-    def update_print_mode_if(self) -> None:
-        if "print-mode" in self.metadata:
-            if f"{self.metadata['print-mode']}".strip() == "no-default":
-                remove = -1
-                for i, p in enumerate(self.printers):
-                    if isinstance(p, StdOutPrinter):
-                        remove = i
-                        break
-                if remove >= 0:
-                    del self.printers[remove]
-            elif f"{self.metadata['print-mode']}".strip() == "default":
-                done = False
-                for i, p in enumerate(self.printers):
-                    if isinstance(p, StdOutPrinter):
-                        done = True
-                        break
-                if not done:
-                    self.printers.append(StdOutPrinter())
-            else:
-                self.logger.warning(
-                    "Incorrect metadata field value 'print-mode': %s",
-                    self.metadata["print-mode"],
-                )
+    @property
+    def logic_mode(self) -> str:
+        return self.metadata.get("logic-mode")
+
+    @property
+    def return_mode(self) -> str:
+        return self.modes.get("return-mode")
+
+    @property
+    def explain_mode(self) -> str:
+        return self.metadata.get("explain-mode")
+
+    @property
+    def print_mode(self) -> str:
+        return self.metadata.get("print-mode")
+
+    @property
+    def unmatched_mode(self) -> str:
+        return self.metadata.get("unmatched-mode")
+
+    # =====================
+
+    @property
+    def transfers(self) -> list[tuple[str, str]]:
+        return self.modes.transfer_mode.transfers
+
+    @property
+    def all_expected_files(self) -> list[str]:
+        return self.modes.files_mode.all_expected_files
+
+    @all_expected_files.setter
+    def all_expected_files(self, efs: list[str]) -> None:
+        self.modes.files_mode.all_expected_files = efs
 
     def _pick_named_path(self, name, *, specific=None) -> str:
         if not self.csvpaths:
@@ -814,10 +783,14 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
             raise InputException("Not a csvpath string")
         scan = ""
         matches = ""
-
+        data = data.strip()
         i = data.find("]")
         if i < 0:
             raise InputException(f"Cannot find the scan part of this csvpath: {data}")
+        if i == len(data) - 1:
+            raise InputException(
+                f"The scan part of this csvpath cannot be last: {data}"
+            )
 
         scan = data[0 : i + 1]
         scan = scan.strip()
@@ -826,12 +799,11 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
         ndata = ndata.strip()
 
         if ndata == "":
-            raise InputException("There must be a match part of this csvpath: {data}")
-
+            raise InputException(f"There must be a match part of this csvpath: {data}")
         if ndata[0] != "[":
-            raise InputException("Cannot find the match part of this csvpath: {data}")
+            raise InputException(f"Cannot find the match part of this csvpath: {data}")
         if ndata[len(ndata) - 1] != "]":
-            raise InputException("The match part of this csvpath is incorrect: {data}")
+            raise InputException(f"The match part of this csvpath is incorrect: {data}")
         matches = ndata
         #
         # if we're given directory(s) to save to, save the parts
@@ -877,6 +849,14 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
     def is_valid(self, tf: bool) -> None:
         self._is_valid = tf
 
+    @property
+    def completed(self) -> bool:
+        if not self.scanner or not self.line_monitor:
+            return False
+        if self.scanner.is_last(self.line_monitor.physical_line_number):
+            return True
+        return False
+
     @property
     def from_line(self):  # pragma: no cover pylint: disable=C0116
         if self.scanner is None:
@@ -927,38 +907,82 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
     # collect(), fast_forward(), and next() are the central methods of CsvPath.
     #
     #
-    def collect(self, nexts: int = -1) -> List[List[Any]]:
+    def collect(
+        self, csvpath: str = None, *, nexts: int = -1, lines=None
+    ) -> List[List[Any]] | LineSpooler:
         """Runs the csvpath forward and returns the matching lines seen as
-        a list of lists"""
+        a list of lists. this method does not holds lines locally, not as
+        accessible attributes. lines are not kept after the run completes
+        and the collected lines are returned.
+
+        the optional lines argument may be an instance of any class that has an append(obj)
+        method. if lines is None, a list is returned.
+        """
+        if self.scanner is None and csvpath is not None:
+            self.parse(csvpath)
         if nexts < -1:
             raise ProcessingException(
                 "Input must be >= -1. -1 means collect to the end of the file."
             )
-        lines = []
+        self.collecting = True
+        #
+        # we're going to use the passed in lines object. if it exists
+        # it is a LineSpooler (we presume). we'll associate it with the
+        # csvpath temporarily so anyone interested can get it. if it
+        # doesn't exist we create a list for the lines var. we create a
+        # list LinesSpooler to handle any inquiries during the run we
+        # do all our line collecting busness with the local lines var
+        # which has an append method regardless of if it is list or
+        # LineSpooler. when we're done we break the link with the csvpath
+        # and return the local variable. if it is a list it is going to
+        # a csvpath user. if it is a LineSpooler it is going to a
+        # CsvPaths instance.
+        #
+        self.lines = lines
+        if lines is None:
+            lines = []
+            self.lines = ListLineSpooler(lines=lines)
         for _ in self.next():
             _ = _[:]
-            lines.append(_)
+            self.lines.append(_)
             if nexts == -1:
                 continue
             if nexts > 1:
                 nexts -= 1
             else:
                 break
+        # we don't want to hold on to data more than needed. but
+        # we do want to return data if we're not spooling. the
+        # way we do that is to keep the local var available with the
+        # list and/or the spooler. the caller needs to be aware of
+        # both possibilities, but both offer __len__ and append.
+        #
+        # we keep the self.lines if it is not a list because that
+        # makes it available to the runtime data collector so we can
+        # see the line count in the metadata, saving opening a
+        # potentially large data.csv to find out how many lines.
+        if isinstance(self.lines, list):
+            self.lines = None
         return lines
 
-    def fast_forward(self) -> None:
+    def fast_forward(self, csvpath=None) -> Self:
         """Runs the path for all rows of the file. Variables are collected
         and side effects like print happen. No lines are collected.
         """
+        if self.scanner is None and csvpath is not None:
+            self.parse(csvpath)
         for _ in self.next():
             pass
+        return self
 
-    def next(self):
+    def next(self, csvpath=None):
         """Iterates over the lines in the CSV file returning those that match
         the csvpath. collect() and fast_forward() call next() behind the scenes.
         """
+        if self.scanner is None and csvpath is not None:
+            self.parse(csvpath)
         start = time.time()
-        if self.run_mode is True:
+        if self.will_run is True:
             for line in self._next_line():
                 b = self._consider_line(line)
                 if b:
@@ -972,6 +996,12 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
                         self.logger.error(msg)
                         raise MatchException(msg)
                     yield line
+                elif self.collecting and self.unmatched_available:
+                    if self.unmatched is None:
+                        self.unmatched = []
+                    line = self.limit_collection(line)
+                    # we aren't None and 0 checking as above. needed?
+                    self.unmatched.append(line)
                 if self.stopped:
                     self.logger.info(
                         "CsvPath has been stopped at line %s",
@@ -1007,14 +1037,12 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
         #
         if self.scanner.filename is None:
             raise FileException("There is no filename")
-        """
-        with open(self.scanner.filename, "r", encoding="utf-8") as file:
-            reader = csv.reader(
-                file, delimiter=self.delimiter, quotechar=self.quotechar
-            )
-            for line in reader:
-        """
-        reader = CsvDataFileReader(
+        #
+        # DataFileReader is abstract. instantiating it results in a concrete subclass.
+        # pylint doesn't like that just because it doesn't see what we're doing.
+        # otoh, is this a bad way to do it? not sure but it works fine.
+        #
+        reader = DataFileReader(  # pylint: disable=E0110
             self.scanner.filename, delimiter=self.delimiter, quotechar=self.quotechar
         )
         for line in reader.next():
@@ -1022,35 +1050,6 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
             yield line
         self.finalize()
 
-    """
-    # potential replacement for method above
-    # this is a proposal for having the results of one csvpath feed into another
-    # in memory. the goal being to both shape the data chain-of-responsibility-style
-    # and also to narrow the data for performance gains.
-    #
-    # we would need:
-    #   - csvpaths.chain_result_data
-    #   - named-path added to csvpath metadata early-on
-    #
-    # caching this here for now. jury is out on if it should be added.
-    #
-    def _next_line_new(self) -> List[Any]:
-        self.logger.info("beginning to scan file: %s", self.scanner.filename)
-        if self.csvpath and self.csvpaths.chain_result_data:
-            rs = csvpath.result_manager.get_named_results(self.metadata["named-paths"])
-            for line in rs[len(rs)-1].lines:
-                yield line
-        elif:
-            with open(self.scanner.filename, "r", encoding="utf-8") as file:
-                reader = csv.reader(
-                    file, delimiter=self.delimiter, quotechar=self.quotechar
-                )
-                for line in reader:
-                    self.track_line(line=line)
-                    yield line
-        self.finalize()
-    """
-
     def finalize(self) -> None:
         """clears caches, etc. this is an internal method, but not _ because
         it is part of the lifecycle and we might find a reason to call it
@@ -1071,6 +1070,8 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
         if self.matcher:
             last_line = self.matcher.line
         self.line_monitor.next_line(last_line=last_line, data=line)
+        if self.line_monitor.physical_line_number == 0:
+            self._run_started_at = datetime.now(timezone.utc)
 
     def _consider_line(self, line):  # pylint: disable=R0912, R0911
         # re: R0912: this method has already been refactored but maybe
@@ -1160,6 +1161,10 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
             return line
         ls = []
         for k in self.limit_collection_to:
+            if k is None or k >= len(line):
+                raise InputException(
+                    f"[{self.identity}] Line {self.line_monitor.physical_line_number}: unknown header name: {k}"
+                )
             ls.append(line[k])
         return ls
 
@@ -1193,12 +1198,26 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
             self.get_total_lines_and_headers()
         return self.line_monitor.physical_end_line_number
 
-    def get_total_lines_and_headers(self) -> int:  # pylint: disable=C0116
-        if self.csvpaths:
-            self.line_monitor = self.csvpaths.file_manager.get_new_line_monitor(
+    def get_total_lines_and_headers(self) -> None:  # pylint: disable=C0116
+        if not self.scanner or not self.scanner.filename:
+            self.logger.error(
+                "Csvpath identified as %s has no filename. Since we could be error handling an exception is not raised.",
+                self.identity,
+            )
+        #
+        # there are times, e.g. when using Lambda, when it may be better to
+        # not use a cache. in the case of Lambda the reason is to avoid working
+        # around the read-only filesystem.
+        #
+        use_cache = self.csvpaths is not None
+        if use_cache:
+            uc = self.csvpaths.config.get(section="cache", name="use_cache")
+            use_cache = uc is None or uc.strip().lower() != "no"
+        if self.csvpaths and use_cache is True:
+            self.line_monitor = self.csvpaths.file_manager.cacher.get_new_line_monitor(
                 self.scanner.filename
             )
-            self.headers = self.csvpaths.file_manager.get_original_headers(
+            self.headers = self.csvpaths.file_manager.cacher.get_original_headers(
                 self.scanner.filename
             )
         else:
@@ -1238,7 +1257,7 @@ class CsvPath(CsvPathPublic, ErrorCollector, Printer):  # pylint: disable=R0902,
             self.matcher = Matcher(
                 csvpath=self, data=self.match, line=line, headers=self.headers, myid=h
             )
-            self.matcher.AND = self._AND
+            self.matcher.AND = self.AND
         else:
             self.logger.debug("Resetting and reloading matcher")
             self.matcher.reset()
