@@ -8,11 +8,9 @@ from csvpath.matching.functions.function import Function
 from csvpath.matching.productions.reference import Reference
 from csvpath.matching.productions.equality import Equality
 from csvpath.matching.util.expression_utility import ExpressionUtility
-from ..util.exceptions import ChildrenException, ChildrenValidationException
 from csvpath.util.config_exception import ConfigurationException
-
-#   from csvpath.util.log_utility import LogUtility
-#   LogUtility.log_brief_trace()
+from csvpath.util.error import ErrorCommsManager
+from ..util.exceptions import ChildrenException
 
 
 class Arg:
@@ -220,11 +218,11 @@ class ArgSet:
         found = len(actuals) == 0
         a = None
         i = 0
-        self._parent._csvpath.logger.debug(
+        self._parent.csvpath.logger.debug(
             "Beginning matches on arg actuals to expected actuals for argset %s",
             self.argset_number,
         )
-        self._parent._csvpath.logger.debug("Actuals: %s", str(actuals))
+        self._parent.csvpath.logger.debug("Actuals: %s", str(actuals))
         for i, a in enumerate(actuals):
             if i >= len(self._args):
                 #
@@ -255,23 +253,8 @@ class ArgSet:
             # in a more practical way.
             #
             # exp!
-            self._parent._csvpath.logger.debug("Checking arg[%i]: %s", i, arg)
-            """
-            if ( not arg or arg.actuals is None ):
-                if self._parent and self._parent._csvpath:
-                    self._parent._csvpath.logger.debug(
-                        "No expectations to validate actual values against in argset {self.argset_number}"
-                    )
-                found = True
-                break
-            if len(arg.actuals) == 0 and len(actuals) > 0:
-                if self._parent and self._parent._csvpath:
-                    self._parent._csvpath.logger.debug(
-                        "No optional args should be presented in {self.argset_number}"
-                    )
-                found = False
-                break
-            """
+            # experiment removed.
+            self._parent.csvpath.logger.debug("Checking arg[%i]: %s", i, arg)
             # end exp
             #
             # start orig w/orig comment:
@@ -280,8 +263,8 @@ class ArgSet:
             # remove the expectations from the args
             #
             if not arg or not arg.actuals or len(arg.actuals) == 0:
-                if self._parent and self._parent._csvpath:
-                    self._parent._csvpath.logger.debug(
+                if self._parent and self._parent.csvpath:
+                    self._parent.csvpath.logger.debug(
                         "No expectations to validate actual values against in argset {self.argset_number}"
                     )
                 found = True
@@ -290,11 +273,11 @@ class ArgSet:
             # end orig
             #
             if Any in arg.actuals:
-                self._parent._csvpath.logger.debug("Found Any so we're done")
+                self._parent.csvpath.logger.debug("Found Any so we're done")
                 found = True
                 continue
             _ = ExpressionUtility.is_one_of(a, arg.actuals)
-            self._parent._csvpath.logger.debug(
+            self._parent.csvpath.logger.debug(
                 "'%s' is_one_of %s returns %s", a, str(arg.actuals), _
             )
             if _ is True:
@@ -303,7 +286,7 @@ class ArgSet:
             found = False
             break
         if not found:
-            self._parent._csvpath.logger.debug(
+            self._parent.csvpath.logger.debug(
                 "%s(%s) not allowed in arg %s of argset %s",
                 type(a),
                 a,
@@ -314,7 +297,7 @@ class ArgSet:
                 f"{type(a)}({a}) not allowed in arg {i + 1} of {len(actuals)}"
             )
         if len(actuals) < self.min_length:
-            self._parent._csvpath.logger.debug(
+            self._parent.csvpath.logger.debug(
                 "Values received %s are too few for argset %s",
                 actuals,
                 self.argset_number,
@@ -353,6 +336,10 @@ class Args:
         self.matched = False
         self._args_match = True
 
+    @property
+    def csvpath(self):
+        return self._csvpath
+
     def reset(self) -> None:
         self._args_match = True
         self.matched = False
@@ -385,7 +372,7 @@ class Args:
             return
         if (
             len(self._argsets) > 0
-            and len(self._argsets[0]._args) == 0
+            and len(self._argsets[0].args) == 0
             and len(siblings) == 0
         ):
             return
@@ -409,7 +396,7 @@ class Args:
             if self._matchable is None:
                 # this should only be testing
                 raise ChildrenException(msg)
-            self._matchable.raiseChildrenException(msg)
+            self._matchable.raise_children_exception(msg)
         self.validated = True
 
     def matches(self, actuals: List[Any]) -> None:
@@ -423,7 +410,7 @@ class Args:
                 f"Cannot have None in {self.matchable.my_chain} because it has the notnone qualifier"
             ]
         else:
-            for aseti, aset in enumerate(self._argsets):
+            for aset in self._argsets:
                 ms = aset.matches(actuals)
                 if len(ms) > 0:
                     mismatch_count += 1
@@ -461,4 +448,15 @@ class Args:
             pm = f"Wrong value in match component {ei}: {pm}"
             lpm = f"{pm}: {mismatches}"
             self._matchable.matcher.csvpath.logger.error(lpm)
-            self._matchable.raiseChildrenException(pm)
+            if (
+                not ErrorCommsManager(
+                    csvpath=self._matchable.matcher.csvpath
+                ).do_i_raise()
+                and self._matchable.matcher.csvpath.match_validation_errors
+            ):
+                # we match on errors so we have to handle and keep going as best we can.
+                pm = self._matchable.decorate_error_message(pm)
+                e = ChildrenException(pm)
+                self._matchable.handle_error(e)
+            else:
+                self._matchable.raise_children_exception(pm)
