@@ -1,9 +1,5 @@
 # pylint: disable=C0114
-from csvpath.matching.util.exceptions import ChildrenException
-from ..function_focus import ValueProducer
-from csvpath.matching.productions import Term, Variable, Header, Reference
-from csvpath.matching.util.expression_utility import ExpressionUtility
-from ..function import Function
+from csvpath.matching.productions import Term
 from ..args import Args
 from .type import Type
 
@@ -59,7 +55,7 @@ class String(Type):
         if maxlen is None:
             maxlen = len(value)
         if maxlen < minlen:
-            self.raiseChildrenException(
+            self.raise_children_exception(
                 "Max length ({maxlen}) cannot be less than min length ({minlen})"
             )
         self.match = minlen <= len(value) <= maxlen
