@@ -147,7 +147,9 @@ class ExpressionUtility:
             return int(v)
         except ValueError as e:
             if should_i_raise is True:
-                raise ValueError("ExpressionUtility cannot convert '{v}' to int") from e
+                raise ValueError(
+                    f"ExpressionUtility cannot convert '{v}' to int"
+                ) from e
             else:
                 return v
 
@@ -418,7 +420,7 @@ class ExpressionUtility:
                 p = p.parent
             else:
                 break
-        return hashlib.sha256(id.encode("utf-8")).hexdigest()
+        return f"_intx_{hashlib.sha256(id.encode('utf-8')).hexdigest()}"
 
     @classmethod
     def my_chain(cls, thing):
