@@ -64,6 +64,7 @@ class Config:
         self.load = load
         self._cache_dir_path = None
         self._function_imports = None
+        self._additional_listeners = None
         self._csvpath_file_extensions = None
         self._csv_file_extensions = None
         self._csvpath_errors_policy = None
@@ -73,6 +74,10 @@ class Config:
         self._log_file = None
         self._log_files_to_keep = None
         self._log_file_size = None
+        self._archive_path = None
+        self._transfer_root = None
+        self._inputs_files_path = None
+        self._inputs_csvpaths_path = None
         self._config = RawConfigParser()
         self.log_file_handler = None
         self._configpath = environ.get(Config.CSVPATH_CONFIG_FILE_ENV)
@@ -102,22 +107,32 @@ class Config:
     def config_path(self) -> str:
         return self._configpath
 
-    def _get(self, section: str, name: str):
+    def get(self, *, section: str, name: str, default=None):
+        return self._get(section, name, default)
+
+    def _get(self, section: str, name: str, default=None):
         if self._config is None:
             raise ConfigurationException("No config object available")
         try:
             s = self._config[section][name]
-            s = s.strip()
             ret = None
             if s.find(",") > -1:
                 ret = [s.strip() for s in s.split(",")]
             else:
-                ret = s
+                ret = s.strip()
             return ret
         except KeyError:
-            print(
-                f"WARNING: Check config at {self.config_path} for [{section}][{name}]"
-            )
+            if self.csvpath_log_level == LogLevels.DEBUG:
+                print(f"Check config at {self.config_path} for [{section}][{name}]")
+            return default
+
+    def add_to_config(self, section, key, value, save_load: bool = True) -> None:
+        if not self._config.has_section(section):
+            self._config.add_section(section)
+        self._config.set(section, key, value)
+        if save_load is True:
+            self.save_config()
+            self._load_config()
 
     def save_config(self) -> None:
         with open(self.configpath, "w") as f:
@@ -142,7 +157,7 @@ extensions = txt, csvpath, csvpaths
 [csv_files]
 extensions = txt, csv, tsv, dat, tab, psv, ssv
 [errors]
-csvpath = raise, collect, stop, fail, print
+csvpath = collect, fail, print
 csvpaths = raise, collect
 [logging]
 csvpath = info
@@ -156,6 +171,64 @@ path =
 imports =
 [cache]
 path =
+
+[listeners]
+# add listener group names to send events to the channel they represent
+groups =
+#slack, marquez, ckan, sftp, sftpplus, otlp
+
+# add otlp to the list of groups above to push observability metrics to an OpenTelemetry endpoint
+otlp.result = from csvpath.managers.integrations.otlp.otlp_result_listener import OpenTelemetryResultListener
+otlp.results = from csvpath.managers.integrations.otlp.otlp_results_listener import OpenTelemetryResultsListener
+
+# add sftp to the list of groups above to push content and metadata to an SFTP account
+sftp.results = from csvpath.managers.integrations.sftp.sftp_listener import SftpListener
+
+# add ckan to the list of groups above to push content and metadata to CKAN
+ckan.results = from csvpath.managers.integrations.ckan.ckan_listener import CkanListener
+
+#add marquez to the list of groups above for OpenLineage events to a Marquez server
+marquez.file = from csvpath.managers.integrations.ol.file_listener_ol import OpenLineageFileListener
+marquez.paths = from csvpath.managers.integrations.ol.paths_listener_ol import OpenLineagePathsListener
+marquez.result = from csvpath.managers.integrations.ol.result_listener_ol import OpenLineageResultListener
+marquez.results = from csvpath.managers.integrations.ol.results_listener_ol import OpenLineageResultsListener
+
+# add slack to the list of groups above for alerts to slack webhooks
+slack.file = from csvpath.managers.integrations.slack.sender import SlackSender
+slack.paths = from csvpath.managers.integrations.slack.sender import SlackSender
+slack.result = from csvpath.managers.integrations.slack.sender import SlackSender
+slack.results = from csvpath.managers.integrations.slack.sender import SlackSender
+
+[sftpplus]
+admin_user = SFTPPLUS_ADMIN_USERNAME
+admin_password = SFTPPLUS_ADMIN_PASSWORD
+server = SFTPPLUS_SERVER
+port = SFTPPLUS_PORT
+
+[ckan]
+server = http://localhost:80
+api_token =
+
+[marquez]
+base_url = http://localhost:5000
+endpoint = api/v1/lineage
+api_key = "none"
+timeout = 5
+verify = False
+
+[slack]
+# add your main webhook here. to set webhooks on a csvpath-by-csvpath basis add
+# on-valid-slack: webhook-minus-'https://' and/or
+# on-invalid-slack: webhook-minus-'https://'
+webhook_url =
+
+[results]
+archive = archive
+transfers = transfers
+[inputs]
+files = inputs/named_files
+csvpaths = inputs/named_paths
+on_unmatched_file_fingerprints = halt
             """
             file.write(c)
             print(f"Created a default config file at {directory} with name {name}.")
@@ -178,7 +251,47 @@ path =
             return dirpath
         return None
 
+    def _assure_archive_path(self) -> None:
+        if self.load:
+            if self.archive_path is None or self.archive_path.strip() == "":
+                self.archive_path = "archive"
+            if self.archive_path.strip().lower().startswith("s3://"):
+                return
+            if not path.exists(self.archive_path):
+                os.makedirs(self.archive_path)
+
+    def _assure_transfer_root(self) -> None:
+        if self.load:
+            if self.transfer_root is None or self.transfer_root.strip() == "":
+                self.transfer_root = "transfers"
+            if not path.exists(self.transfer_root):
+                os.makedirs(self.transfer_root)
+
+    def _assure_inputs_files_path(self) -> None:
+        if self.load:
+            if self.inputs_files_path is None or self.inputs_files_path.strip() == "":
+                self.inputs_files_path = "inputs/named_files"
+            if self.inputs_files_path.strip().lower().startswith("s3://"):
+                return
+            if not path.exists(self.inputs_files_path):
+                os.makedirs(self.inputs_files_path)
+
+    def _assure_inputs_csvpaths_path(self) -> None:
+        if self.load:
+            if (
+                self.inputs_csvpaths_path is None
+                or self.inputs_csvpaths_path.strip() == ""
+            ):
+                self.inputs_csvpaths_path = "inputs/named_paths"
+            if self.inputs_csvpaths_path.strip().lower().startswith("s3://"):
+                return
+            if not path.exists(self.inputs_csvpaths_path):
+                os.makedirs(self.inputs_csvpaths_path)
+
     def _assure_cache_path(self) -> None:
+        uc = self.get(section="cache", name="use_cache")
+        if uc and uc.strip().lower() == "no":
+            return
         if self.load:
             if self.cache_dir_path is None or self.cache_dir_path.strip() == "":
                 self.cache_dir_path = "cache"
@@ -276,7 +389,7 @@ path =
         ):
             raise ConfigurationException("CsvPath error policy is wrong")
         for _ in self.csvpath_errors_policy:
-            if _ not in OnError:
+            if _ not in [s.value for s in OnError]:
                 raise ConfigurationException(f"CsvPath error policy {_} is wrong")
         if (
             self.csvpaths_errors_policy is None
@@ -285,7 +398,7 @@ path =
         ):
             raise ConfigurationException("CsvPaths error policy is wrong")
         for _ in self.csvpaths_errors_policy:
-            if _ not in OnError:
+            if _ not in [s.value for s in OnError]:
                 raise ConfigurationException(f"CsvPaths error policy {_} is wrong")
         #
         # log levels
@@ -296,13 +409,13 @@ path =
             raise ConfigurationException(
                 f"CsvPath log level is wrong: {self.csvpath_log_level}"
             )
-        if self.csvpath_log_level not in LogLevels:
+        if self.csvpath_log_level not in [s.value for s in LogLevels]:
             raise ConfigurationException(f"CsvPath log level {_} is wrong")
         if self.csvpaths_log_level is None or not isinstance(
             self.csvpaths_log_level, str
         ):
             raise ConfigurationException("CsvPaths log level is wrong")
-        if self.csvpaths_log_level not in LogLevels:
+        if self.csvpaths_log_level not in [s.value for s in LogLevels]:
             raise ConfigurationException(f"CsvPaths log level {_} is wrong")
         #
         # log files config
@@ -326,6 +439,11 @@ path =
         # default config, but regardless, we create the dir.
         #
         self._assure_cache_path()
+        #
+        # make sure a inputs dirs exist.
+        #
+        self._assure_inputs_files_path()
+        self._assure_inputs_csvpaths_path()
 
     # ======================================
 
@@ -337,6 +455,23 @@ path =
     def configpath(self, path: str) -> None:
         self._configpath = path
 
+    def additional_listeners(self, listener_type) -> list[str]:
+        # pull type for group names
+        # for each group name find:
+        #    listener_type.groupname=listner
+        listeners = []
+        groups = self._get("listeners", "groups")
+        if groups is None:
+            groups = []
+        if isinstance(groups, str):
+            groups = [groups]
+        for group in groups:
+            lst = f"{group}.{listener_type}"
+            listener = self._get("listeners", lst)
+            if listener is not None:
+                listeners.append(listener)
+        return listeners
+
     @property
     def cache_dir_path(self) -> str:
         return self._cache_dir_path
@@ -345,6 +480,73 @@ path =
     def cache_dir_path(self, p) -> None:
         self._cache_dir_path = p
 
+    def halt_on_unmatched_file_fingerprints(self) -> bool:
+        houf = self._get("inputs", "on_unmatched_file_fingerprints")
+        if houf == "halt":
+            return True
+        elif houf == "continue":
+            return False
+        return None
+
+    @property
+    def transfer_root(self) -> str:
+        if self._transfer_root is None:
+            self._transfer_root = self._get("results", "transfers")
+            if self._transfer_root is None:
+                self._transfer_root = "transfers"
+                self.add_to_config("results", "transfers", "transfers")
+        return self._transfer_root
+
+    @transfer_root.setter
+    def transfer_root(self, p) -> None:
+        self._transfer_root = p
+
+    @property
+    def archive_path(self) -> str:
+        if self._archive_path is None:
+            self._archive_path = self._get("results", "archive")
+            if self._archive_path is None:
+                self._archive_path = "archive"
+                self.add_to_config("results", "archive", "archive")
+        return self._archive_path
+
+    @archive_path.setter
+    def archive_path(self, p) -> None:
+        self._archive_path = p
+
+    @property
+    def archive_name(self) -> str:
+        p = self.archive_path
+        if p.find(os.sep) > -1:
+            p = p[p.rfind(os.sep) + 1 :]
+        return p
+
+    @property
+    def inputs_files_path(self) -> str:
+        if self._inputs_files_path is None:
+            self._inputs_files_path = self._get("inputs", "files")
+            if self._inputs_files_path is None:
+                self._inputs_files_path = "inputs"
+                self.add_to_config("inputs", "files", "inputs/named_files")
+        return self._inputs_files_path
+
+    @inputs_files_path.setter
+    def inputs_files_path(self, p) -> None:
+        self._inputs_files_path = p
+
+    @property
+    def inputs_csvpaths_path(self) -> str:
+        if self._inputs_csvpaths_path is None:
+            self._inputs_csvpaths_path = self._get("inputs", "csvpaths")
+            if self._inputs_csvpaths_path is None:
+                self._inputs_csvpaths_path = "inputs"
+                self.add_to_config("inputs", "csvpaths", "inputs/named_paths")
+        return self._inputs_csvpaths_path
+
+    @inputs_csvpaths_path.setter
+    def inputs_csvpaths_path(self, p) -> None:
+        self._inputs_csvpaths_path = p
+
     @property
     def function_imports(self) -> str:
         return self._function_imports
